"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _Document, _Document$prototype;

var _default = createNodeIterator;
exports["default"] = _default;
var create = Object.create.bind(Object);
var getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor.bind(Object);
var getPrototypeOf = Object.getPrototypeOf.bind(Object);
var original = (_Document = Document) === null || _Document === void 0 ? void 0 : (_Document$prototype = _Document.prototype) === null || _Document$prototype === void 0 ? void 0 : _Document$prototype.createNodeIterator;
var originalBound = original && Function.prototype.call.bind(original);

function createNodeIterator(root) {
  var whatToShow = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0xFFFFFFFF;
  var filter = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
  var document = this;

  if (typeof originalBound === 'function') {
    var iter = originalBound(document, root, whatToShow, filter, false);
    var prototype = getPrototypeOf(iter);
    var referenceNode = getOwnPropertyDescriptor(prototype, 'referenceNode');
    return referenceNode === undefined ? wrapNodeIterator(iter) : iter;
  }

  return implementNodeIterator(root, whatToShow, filter);
}

function successor(node, root) {
  if (node.firstChild) return node.firstChild;

  while (node !== root) {
    if (node.nextSibling) return node.nextSibling;
    node = node.parentNode;
  }

  return null;
}

function predecessor(node, root) {
  if (node === root) return null;

  if (node.previousSibling) {
    node = node.previousSibling;

    while (node.lastChild) {
      node = node.lastChild;
    }

    return node;
  }

  return node.parentNode;
}

function implementNodeIterator(root, whatToShow, filter) {
  var data = {
    referenceNode: root,
    pointerBeforeReferenceNode: true
  };

  function _filter(node) {
    return filter ? filter(node) === 1 : true;
  }

  function _show(node) {
    return whatToShow >> node.nodeType - 1 & 1 === 1;
  }

  return create({}, {
    root: {
      value: root
    },
    referenceNode: {
      get: function referenceNode() {
        return data.referenceNode;
      }
    },
    pointerBeforeReferenceNode: {
      get: function pointerBeforeReferenceNode() {
        return data.pointerBeforeReferenceNode;
      }
    },
    whatToShow: {
      value: whatToShow
    },
    filter: {
      value: filter
    },
    nextNode: {
      value: function nextNode() {
        var node = data.referenceNode;
        var before = data.pointerBeforeReferenceNode;

        while (true) {
          if (before) before = false;else node = successor(node, root);
          if (node === null) return null;
          if (_show(node) && _filter(node)) break;
        }

        data.referenceNode = node;
        data.pointerBeforeReferenceNode = before;
        return node;
      }
    },
    previousNode: {
      value: function previousNode() {
        var before = data.pointerBeforeReferenceNode;
        var node = data.referenceNode;

        while (true) {
          if (!before) before = true;else node = predecessor(node, data.root);
          if (node === null) return null;
          if (_show(node) && _filter(node)) break;
        }

        data.referenceNode = node;
        data.pointerBeforeReferenceNode = before;
        return node;
      }
    },
    detach: {
      value: function detach() {}
    },
    toString: {
      value: function toString() {
        return '[object NodeIterator]';
      }
    }
  });
}

function wrapNodeIterator(iter) {
  var data = {
    referenceNode: iter.root,
    pointerBeforeReferenceNode: true
  };
  var prototype = getPrototypeOf(iter);
  var wrapperPrototype = create(prototype, {
    root: {
      get: function root() {
        return iter.root;
      }
    },
    referenceNode: {
      get: function referenceNode() {
        return data.referenceNode;
      }
    },
    pointerBeforeReferenceNode: {
      get: function pointerBeforeReferenceNode() {
        return data.pointerBeforeReferenceNode;
      }
    },
    whatToShow: {
      get: function whatToShow() {
        return iter.whatToShow;
      }
    },
    filter: {
      get: function filter() {
        return iter.filter;
      }
    },
    nextNode: {
      value: function nextNode() {
        var result = iter.nextNode();
        if (result === null) return null;
        data.referenceNode = result;
        data.pointerBeforeReferenceNode = false;
        return data.referenceNode;
      }
    },
    previousNode: {
      value: function previousNode() {
        var result = iter.previousNode();
        if (result === null) return null;
        data.referenceNode = result;
        data.pointerBeforeReferenceNode = true;
        return data.referenceNode;
      }
    },
    detach: {
      value: function detach() {}
    },
    toString: {
      value: function toString() {
        return '[object NodeIterator]';
      }
    }
  });
  return create(wrapperPrototype);
}
//# sourceMappingURL=implementation.js.map